//+---------------------------------------------------------------------------+
//|   EA VERSION
//|   MTF_RenkoLiveChart_v1.0.mq4
//|   Original coded by LV & Lou G Inspired from Renko script by "e4" (renko_live_scr.mq4)
//|   Copyleft 2009 LastViking
//|   Expanded on more Renko TF, more refreshing offline charts for each of them and global variable communication by laro_ron
//|   
//|   
//|   Aug 12 2009 (LV): 
//|            - Wanted volume in my Renko chart so I wrote my own script
//|     
//|   Aug 20-21 2009 (LV) (v1.1 - v1.3):
//|            - First attempt at live Renko brick formation (bugs O bugs...)
//|            - Fixed problem with strange symbol names at some 5 digit 
//|               brokers (credit to Tigertron)
//|     
//|   Aug 24 2009 (LV) (v1.4):
//|            - Handle High / Low in history in a reasonable way (prev. 
//|               used Close)
//|   
//|   Aug 26 2009 (Lou G) (v1.5/v1.6):
//|            - Finaly fixing the "late appearance" (live Renko brick 
//|               formation) bug
//| 
//|   Aug 31 2009 (LV) (v2.0):
//|            - Not a script anylonger, but run as indicator 
//|            - Naroved down the MT4 bug that used to cause the "late appearance bug" 
//|               a little closer (has to do with High / Low gaps)
//|            - Removed the while ... sleep() loop. Renko chart is now tick 
//|               driven: -MUSH nicer to system resources this way
//| 
//|   Sep 03 2009 (LV) (v2.1):
//|            - Fixed so that Time[] holds the open time of the renko 
//|               bricks (prev. used time of close)
//|     
//|   Sep 16 2009 (Lou G) (v3.0): 
//|            - Optional wicks added
//|            - Conversion back to EA 
//|            - Auto adjust for 5 and 6 dec brokers added
//|               enter RenkoBoxSize as "actual" size e.g. "10" for 10 pips
//|            - Compensation for "zero compare" problem added
//|
//|   Okt 05 2009 (LV) (v3.1): 
//|            - Fixed a bug related to BoxOffset
//|            - Auto adjust for 3 and 4 dec JPY pairs
//|            - Removed init() function
//|            - Changed back to old style Renko brick formation
//| 
//|   Okt 13 2009 (LV) (v3.2): 
//|            - Added "EmulateOnLineChart" option (credit to Skipperxit/Mimmo)
//| 
//|   Apr 23 2012 (laro_ron) (MTF_RenkoLiveChart_v1.0): 
//|            - save the functionality RLC v3.2
//|            - max 3 Renko TF (expanded to 5) or more by recoding
//|            - Refresh all generated Renko Time Frame charts (for multiple offline chart also)
//|            - global variable communication with own EAs and indi
//+---------------------------------------------------------------------------+

/* Description of MTF version Apr 23 2012
EExpanded for more Renko TF and more offline charts for each of them and communication with your own indicators/EAs by global variables
The code is derived and refined from original Okt 13 2009 (LV) (v3.2)

Main code is separated from start() to one_start()
The original functionality is preserved
All names of the variable are saved using #define by similar name in the arrays[][]
If RenkoTimeFrame_1 = 0 start numbering Renko TF from 16
If any RenkoBoxSize is 0 init() doesn't check more RenkoBoxSize
Each Renko TF refreshes more than 1st offline chart (refreshses all of them)
Information about opened offline chart by Comment
The code is prepared for extend to 5 Renko TF
Coded modular for easy expanding more than 5 Renko TF by _dim_renko
The externs RenkoBoxOffset and EmulateOnLineChart are common for all charts (optional)
Common externs are prepared to change to regular externs (for each Renko one extern)
Idea of the code for multiple refresh is separated and recoded from v1.9 P4L PeriodCon.mq4 by pips4life of ForexFactory.com
SendTick = true will simulate an incoming tick so you can create offline charts when the market is closed

If no offline chart is opened for renko TF the function ArrayCopyRates() don't work properly (any refresh) : MT4 behaviour
So if use ArrayCopyRates() to get the array rates[][] of Renko TF in regular TF, MIN ONE OFFLINE RENKO CHART HAVE TO BE OPEN FOR SELECTED RENKO TF
The information about opened offline charts (refreshed) are written on the chart and 
set as global variable with the name "MTF_RLC_" + SymbolName + "_" + RenkoTimeFrame whith the contents : 
1 : mean offline chart is refreshing
0 : mean offline chart is not refreshing
Your own indicator can check validity of actual data in HST by global variable before uses ArrayCopyRates()
Check globals with F3 to see how it works

Next advantage of MTF version over original is that all Renko offline TF get refresh by the same tick from broker (one run MTF_RLC) and is prepared for reading HST 

by MQL ArrayCopyRates()
The speed and consumtion of CPU is better than multiple run RLC v3.2
Inform me please about significant inevitable changes in the code or bugs. 
I use HST output of MTF_RenkoLiveChart in my own indicators working in the regular online TF for natural S/R levels. 

HST read by : if (glob_RenkoTimeFrame == 1.0) ArrayCopyRates(rates, NULL, RenkoTimeFrame). 
Thanks.  laro_ron
*/


#property copyright "2012 laro_ron by LV, Lou G and pips4life"
#property link      "la.ron.29@gmail.com" 

#include <WinUser32.mqh>
#include <stdlib.mqh>
//+------------------------------------------------------------------+
#import "user32.dll"
	int RegisterWindowMessageA(string lpString); 
   int GetParent(int hWnd);
#import

// --------------------------------------- extern
extern int RenkoBoxSize_1 = 10;               // box size for renko candle
extern int RenkoBoxSize_2 = 10;
extern int RenkoBoxSize_3 = 10;
extern int RenkoBoxSize_4 = 10;
//extern int RenkoBoxSize_5 = 0;

extern int RenkoTimeFrame_1 = 10;             // Time frame for the offline renko chart (default 16)
extern int RenkoTimeFrame_2 = 11;             // Time frame for the offline renko chart (default ++)
extern int RenkoTimeFrame_3 = 12;             // Time frame for the offline renko chart (default ++)
extern int RenkoTimeFrame_4 = 13;             // Time frame for the offline renko chart (default ++)
//extern int RenkoTimeFrame_5 = 0;             // Time frame for the offline renko chart (default ++)

extern bool ShowWicks_1 = false;             // true : save High and Low to HST
extern bool ShowWicks_2 = false;
extern bool ShowWicks_3 = false;
extern bool ShowWicks_4 = false;
//extern bool ShowWicks_5 = false;

int RenkoBoxOffset = 0;               // offset from round renko level - common parameter - you can use specific extern for each chart and change init()
bool EmulateOnLineChart = true;       // "Refresh" menu item - reprocessing of the history - common parameter - you can use specific extern for each chart and change init()

//SendTick = true will simulate an incoming tick so you can create offline charts when the market is closed
extern bool SendTick = false;                 // send Internal Message simulate an incoming tick (I just fulfill the functionality of RLC v3.2)
extern bool StrangeSymbolName = false;       // true : get first 6 char only

int RenkoBoxOffset_1;                  // possible extern offset from round renko level
int RenkoBoxOffset_2;                  // possible extern
int RenkoBoxOffset_3;                  // possible extern
int RenkoBoxOffset_4;                  // possible extern
//int RenkoBoxOffset_5;                  // possible extern

bool EmulateOnLineChart_1;             // possible extern "Refresh" menu item - reprocessing of the history
bool EmulateOnLineChart_2;             // possible extern
bool EmulateOnLineChart_3;             // possible extern
bool EmulateOnLineChart_4;             // possible extern
//bool EmulateOnLineChart_5;             // possible extern

// --------------------------------------- define
#define _dim_renko            5           // max numbers of RenkoLiveCharts (dimension for work arrays)

// int extern static variables and globals
#define _RenkoBoxSize         0
#define _RenkoBoxOffset       1
#define _RenkoTimeFrame       2
#define _HstHandle            3
#define _dim_ie               4           // 2nd dimension  integer externs

// bool extern static variables and globals
#define _ShowWicks            0
#define _EmulateOnLineChart   1
#define _ChartOpened          2           // true : offline chart opened  ,  false : not opened
#define _dim_be               3           // 2nd dimension bool externs

#define  CHART_CMD_UPDATE_DATA            33324    // 0x822C  emulate the user selecting the "Refresh" menu item - reprocessing of the history

// --------------------------------------- globals
string base_glob;                         // base for globals variables
int MT4InternalMsg;
string SymbolName;                        // index
int N;                                    // number of creating Renko Live Charts in the HST

int extint[_dim_renko][_dim_ie];          // extern int parameters  RenkoBoxSize , RenkoBoxOffset , RenkoTimeFrame , HstHandle
bool extbool[_dim_renko][_dim_be];        // extern bool parameters   ShowWicks , EmulateOnLineChart , ChartOpened



// initialization of arrays by extern variables
void init()
{
   if(StrangeSymbolName) SymbolName = StringSubstr(Symbol(), 0, 6);
   else SymbolName = Symbol();
   N = 0;
   base_glob = "MTF_RLC_" + SymbolName + "_";
   if (SendTick) MT4InternalMsg = RegisterWindowMessageA("MetaTrader4_Internal_Message");      // simulate an incoming tick
   
   RenkoBoxOffset_1 = RenkoBoxOffset;           // remove line if extern exists
   RenkoBoxOffset_2 = RenkoBoxOffset;           // remove line if extern exists
   RenkoBoxOffset_3 = RenkoBoxOffset;           // remove line if extern exists
   RenkoBoxOffset_4 = RenkoBoxOffset;           // remove line if extern exists
//   RenkoBoxOffset_5 = RenkoBoxOffset;           // remove line if extern exists

   EmulateOnLineChart_1 = EmulateOnLineChart;   // remove line if extern exists
   EmulateOnLineChart_2 = EmulateOnLineChart;   // remove line if extern exists
   EmulateOnLineChart_3 = EmulateOnLineChart;   // remove line if extern exists
   EmulateOnLineChart_4 = EmulateOnLineChart;   // remove line if extern exists
//   EmulateOnLineChart_5 = EmulateOnLineChart;   // remove line if extern exists


// 1. offline renko chart
   extint[N][_RenkoBoxSize]            = RenkoBoxSize_1;
   extint[N][_RenkoBoxOffset]          = RenkoBoxOffset_1;
   if (RenkoTimeFrame_1 == 0) extint[N][_RenkoTimeFrame] = 16;       // default Renko TF begin from 16
   else extint[N][_RenkoTimeFrame]     = RenkoTimeFrame_1;
   extbool[N][_ShowWicks]              = ShowWicks_1;
   extbool[N][_EmulateOnLineChart]     = EmulateOnLineChart_1;
   extint[N][_HstHandle]               = -1;
   N++;

// 2. offline renko chart
   if (RenkoBoxSize_2 == 0) return;
   extint[N][_RenkoBoxSize]            = RenkoBoxSize_2;
   extint[N][_RenkoBoxOffset]          = RenkoBoxOffset_2;
   if (RenkoTimeFrame_2 == 0) extint[N][_RenkoTimeFrame] = extint[N-1][_RenkoTimeFrame] + 1;    // increased
   else extint[N][_RenkoTimeFrame]     = RenkoTimeFrame_2;
   extbool[N][_ShowWicks]              = ShowWicks_2;
   extbool[N][_EmulateOnLineChart]     = EmulateOnLineChart_2;
   extint[N][_HstHandle]               = -1;
   N++;
   
// 3. offline renko chart
   if (RenkoBoxSize_3 == 0) return;
   extint[N][_RenkoBoxSize]            = RenkoBoxSize_3;
   extint[N][_RenkoBoxOffset]          = RenkoBoxOffset_3;
   if (RenkoTimeFrame_3 == 0) extint[N][_RenkoTimeFrame] = extint[N-1][_RenkoTimeFrame] + 1;    // increased
   else extint[N][_RenkoTimeFrame]     = RenkoTimeFrame_3;
   extbool[N][_ShowWicks]              = ShowWicks_3;
   extbool[N][_EmulateOnLineChart]     = EmulateOnLineChart_3;
   extint[N][_HstHandle]               = -1;
   N++;

// 4. offline renko chart
   if (RenkoBoxSize_4 == 0) return;
   extint[N][_RenkoBoxSize]            = RenkoBoxSize_4;
   extint[N][_RenkoBoxOffset]          = RenkoBoxOffset_4;
   if (RenkoTimeFrame_4 == 0) extint[N][_RenkoTimeFrame] = extint[N-1][_RenkoTimeFrame] + 1;    // increased
   else extint[N][_RenkoTimeFrame]     = RenkoTimeFrame_4;
   extbool[N][_ShowWicks]              = ShowWicks_4;
   extbool[N][_EmulateOnLineChart]     = EmulateOnLineChart_4;
   extint[N][_HstHandle]               = -1;
   N++;

/* unhide and extend the appropriate externs
   
// 5. offline renko chart
   if (RenkoBoxSize_5 == 0) return;
   extint[N][_RenkoBoxSize]            = RenkoBoxSize_5;
   extint[N][_RenkoBoxOffset]          = RenkoBoxOffset_5;
   if (RenkoTimeFrame_5 == 0) extint[N][_RenkoTimeFrame] = extint[N-1][_RenkoTimeFrame] + 1;    // increased
   else extint[N][_RenkoTimeFrame]     = RenkoTimeFrame_5;
   extbool[N][_ShowWicks]              = ShowWicks_5;
   extbool[N][_EmulateOnLineChart]     = EmulateOnLineChart_5;
   extint[N][_HstHandle]               = -1;
   N++;
   
*/

// for more RenkoLiveChart increase _dim_renko (real limit for the _dim_renko is "unlimited") and spread the code
//#define _dim_renko            5            // max numbers of RenkoLiveCharts (dimension for work arrays)
}
//
//


// ---------------------------------------------------------------------------------------------------------------------------------
// start point if new tick is comming
int start()
{
   for (int i=0; i<N; i++) one_start(extint, extbool, i);                     // do it for all Renko charts
   com(extint, extbool);                                                      // comment information about offline chart
}
// ---------------------------------------------------------------------------------------------------------------------------------


// if new parameters of MTF_RenkoLiveChart or EA removed from the chart
int deinit()
{
   for (int i=0; i<N; i++) {
      if(extint[i][_HstHandle] > -1) {
         FileClose(extint[i][_HstHandle]);
         extint[i][_HstHandle] = -1;
      }
   }	  
   Comment("");
   GlobalVariablesDeleteAll(base_glob);
   return(0);
}

// process one Renko TF
void one_start(int& exti[][],           // extern int parameters  RenkoBoxSize , RenkoBoxOffset , RenkoTimeFrame , HstHandle
               bool& extb[][],          // extern bool parameters   ShowWicks , EmulateOnLineChart , ChartOpened
               int n)                   // number of Renko TF
{   
#define _BoxPoints   0     // double static variables
#define _UpWick      1
#define _DnWick      2
#define _PrevLow     3
#define _PrevHigh    4
#define _PrevOpen    5
#define _PrevClose   6
#define _CurVolume   7
#define _CurLow      8
#define _CurHigh     9
#define _CurOpen     10
#define _CurClose    11
#define _dim_dv      12    // 2nd dimension double static variables
   
#define _PrevTime    0     // integer static variables
#define _LastFPos    1
#define _BoxSize     2
#define _BoxOffset   3
#define _dim_iv      4     // 2nd dimension integer static variables

   static double dv[_dim_renko][_dim_dv];    // double local static variables from _v3.3 :    BoxPoints, UpWick, DnWick, PrevLow, PrevHigh, PrevOpen, PrevClose, CurVolume, CurLow, CurHigh, CurOpen, CurClose
   static int iv[_dim_renko][_dim_iv];       // integer local static variables from _v3.3 :   PrevTime, LastFPos, BoxSize, BoxOffset
   int i;
   
	//+------------------------------------------------------------------+

	// Start : Initialization + open HST 
	// This is only executed ones, then the first tick arives.
	if(exti[n][_HstHandle] < 0) {
		// Init

		// Error checking	
		if(!IsConnected()) {
			Print("Waiting for connection...");
			return(0);
		}							
		if(!IsDllsAllowed()) {
			Alert("Error: Dll calls must be allowed!");
			return(-1);
		}
				
		if(MathAbs(exti[n][_RenkoBoxOffset]) >= exti[n][_RenkoBoxSize]) {
			Print("Error: RenkoBoxOffset_", n, " should be less then RenkoBoxSize_", n);
			return(-1);
		}
		switch(exti[n][_RenkoTimeFrame]) {
		case 1: case 5: case 15: case 30: case 60: case 240:
		case 1440: case 10080: case 43200: case 0:
			Print("Error: Invald time frame M", exti[n][_RenkoTimeFrame], " used for offline renko chart RenkoTimeFrame_", n, "  (Don\'t use regular TF !)");
			return(-1);
		}
		//
		
		iv[n][_BoxSize] = exti[n][_RenkoBoxSize];
		iv[n][_BoxOffset] = exti[n][_RenkoBoxOffset];
		if(Digits == 5 || (Digits == 3 && StringFind(Symbol(), "JPY") != -1)) {
			iv[n][_BoxSize] = iv[n][_BoxSize]*10;
			iv[n][_BoxOffset] = iv[n][_BoxOffset]*10;
		}
		if(Digits == 6 || (Digits == 4 && StringFind(Symbol(), "JPY") != -1)) {
			iv[n][_BoxSize] = iv[n][_BoxSize]*100;		
			iv[n][_BoxOffset] = iv[n][_BoxOffset]*100;
		}
		
//      if(extb[n][_StrangeSymbolName]) SymbolName = StringSubstr(Symbol(), 0, 6);     moved to init()
//      else SymbolName = Symbol();
	
		dv[n][_BoxPoints] = NormalizeDouble(iv[n][_BoxSize]*Point, Digits);
		dv[n][_PrevLow] = NormalizeDouble(iv[n][_BoxOffset]*Point + MathFloor(Close[Bars-1]/dv[n][_BoxPoints])*dv[n][_BoxPoints], Digits);
		dv[n][_DnWick] = dv[n][_PrevLow];
		dv[n][_PrevHigh] = dv[n][_PrevLow] + dv[n][_BoxPoints];
		dv[n][_UpWick] = dv[n][_PrevHigh];
		dv[n][_PrevOpen] = dv[n][_PrevLow];
		dv[n][_PrevClose] = dv[n][_PrevHigh];
		dv[n][_CurVolume] = 1;
		iv[n][_PrevTime] = Time[Bars-1];
	
		// create / open hst file		
		exti[n][_HstHandle] = FileOpenHistory(SymbolName + exti[n][_RenkoTimeFrame] + ".hst", FILE_BIN|FILE_WRITE);
		if(exti[n][_HstHandle] < 0) {
			Print("Error: can\'t create / open history file: " + ErrorDescription(GetLastError()) + ": " + SymbolName + exti[n][_RenkoTimeFrame] + ".hst");
			return(-1);
		}
		//
   	
		// write hst file header
		int HstUnused[13];
		FileWriteInteger(exti[n][_HstHandle], 400, LONG_VALUE); 			  // Version
		FileWriteString(exti[n][_HstHandle], "", 64);					     // Copyright
		FileWriteString(exti[n][_HstHandle], SymbolName, 12);			     // Symbol
		FileWriteInteger(exti[n][_HstHandle], exti[n][_RenkoTimeFrame], LONG_VALUE);	// Period
		FileWriteInteger(exti[n][_HstHandle], Digits, LONG_VALUE);		 // Digits
		FileWriteInteger(exti[n][_HstHandle], 0, LONG_VALUE);           // Time Sign
		FileWriteInteger(exti[n][_HstHandle], 0, LONG_VALUE);           // Last Sync
		FileWriteArray(exti[n][_HstHandle], HstUnused, 0, 13);          // Unused
		//
   	
 		// process historical data
  		i = Bars-2;
//Print(Symbol() + " " + High[i] + " " + Low[i] + " " + Open[i] + " " + Close[i]);
		//---------------------------------------------------------------------------
  		while(i >= 0) {
  		
			dv[n][_CurVolume] = dv[n][_CurVolume] + Volume[i];
		
			dv[n][_UpWick] = MathMax(dv[n][_UpWick], High[i]);
			dv[n][_DnWick] = MathMin(dv[n][_DnWick], Low[i]);

			// update low before high or the reverse depending on previous bar
			bool UpTrend = High[i]+Low[i] > High[i+1]+Low[i+1];
		
			while(!UpTrend && (Low[i] < dv[n][_PrevLow]-dv[n][_BoxPoints] || CompareDoubles(Low[i], dv[n][_PrevLow]-dv[n][_BoxPoints]))) {
  				dv[n][_PrevHigh]  = dv[n][_PrevHigh] - dv[n][_BoxPoints];
  				dv[n][_PrevLow]   = dv[n][_PrevLow] - dv[n][_BoxPoints];
  				dv[n][_PrevOpen]  = dv[n][_PrevHigh];
  				dv[n][_PrevClose] = dv[n][_PrevLow];

				FileWriteInteger(exti[n][_HstHandle], iv[n][_PrevTime], LONG_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevOpen], DOUBLE_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevLow], DOUBLE_VALUE);

				if(extb[n][_ShowWicks] && dv[n][_UpWick] > dv[n][_PrevHigh]) FileWriteDouble(exti[n][_HstHandle], dv[n][_UpWick], DOUBLE_VALUE);
				else FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevHigh], DOUBLE_VALUE);
				  				  			
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevClose], DOUBLE_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_CurVolume], DOUBLE_VALUE);
				
				dv[n][_UpWick] = 0;
				dv[n][_DnWick] = EMPTY_VALUE;
				dv[n][_CurVolume] = 0;
				dv[n][_CurHigh] = dv[n][_PrevLow];
				dv[n][_CurLow] = dv[n][_PrevLow];  
				
				if(iv[n][_PrevTime] < Time[i]) iv[n][_PrevTime] = Time[i];
				else iv[n][_PrevTime]++;
			}
		
			while(High[i] > dv[n][_PrevHigh]+dv[n][_BoxPoints] || CompareDoubles(High[i], dv[n][_PrevHigh]+dv[n][_BoxPoints])) {
  				dv[n][_PrevHigh] = dv[n][_PrevHigh] + dv[n][_BoxPoints];
  				dv[n][_PrevLow] = dv[n][_PrevLow] + dv[n][_BoxPoints];
  				dv[n][_PrevOpen] = dv[n][_PrevLow];
  				dv[n][_PrevClose] = dv[n][_PrevHigh];
  			
				FileWriteInteger(exti[n][_HstHandle], iv[n][_PrevTime], LONG_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevOpen], DOUBLE_VALUE);

            if(extb[n][_ShowWicks] && dv[n][_DnWick] < dv[n][_PrevLow]) FileWriteDouble(exti[n][_HstHandle], dv[n][_DnWick], DOUBLE_VALUE);
				else FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevLow], DOUBLE_VALUE);
  				  			
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevHigh], DOUBLE_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevClose], DOUBLE_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_CurVolume], DOUBLE_VALUE);
				
				dv[n][_UpWick] = 0;
				dv[n][_DnWick] = EMPTY_VALUE;
				dv[n][_CurVolume] = 0;
				dv[n][_CurHigh] = dv[n][_PrevHigh];
				dv[n][_CurLow] = dv[n][_PrevHigh];  
				
				if(iv[n][_PrevTime] < Time[i]) iv[n][_PrevTime] = Time[i];
				else iv[n][_PrevTime]++;
			}
		
			while(UpTrend && (Low[i] < dv[n][_PrevLow]-dv[n][_BoxPoints] || CompareDoubles(Low[i], dv[n][_PrevLow]-dv[n][_BoxPoints]))) {
  				dv[n][_PrevHigh]  = dv[n][_PrevHigh] - dv[n][_BoxPoints];
  				dv[n][_PrevLow]   = dv[n][_PrevLow] - dv[n][_BoxPoints];
  				dv[n][_PrevOpen]  = dv[n][_PrevHigh];
  				dv[n][_PrevClose] = dv[n][_PrevLow];
  			
				FileWriteInteger(exti[n][_HstHandle], iv[n][_PrevTime], LONG_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevOpen], DOUBLE_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevLow], DOUBLE_VALUE);
				
				if(extb[n][_ShowWicks] && dv[n][_UpWick] > dv[n][_PrevHigh]) FileWriteDouble(exti[n][_HstHandle], dv[n][_UpWick], DOUBLE_VALUE);
				else FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevHigh], DOUBLE_VALUE);
				
				FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevClose], DOUBLE_VALUE);
				FileWriteDouble(exti[n][_HstHandle], dv[n][_CurVolume], DOUBLE_VALUE);

				dv[n][_UpWick] = 0;
				dv[n][_DnWick] = EMPTY_VALUE;
				dv[n][_CurVolume] = 0;
				dv[n][_CurHigh] = dv[n][_PrevLow];
				dv[n][_CurLow] = dv[n][_PrevLow];  
								
				if(iv[n][_PrevTime] < Time[i]) iv[n][_PrevTime] = Time[i];
				else iv[n][_PrevTime]++;
			}		
			i--;
		} 
		iv[n][_LastFPos] = FileTell(exti[n][_HstHandle]);   // Remember Last pos in file
		//

//      Comment("RenkoLiveChart(" + exti[n][_RenkoBoxSize] + "): Open Offline ", SymbolName, ",M", exti[n][_RenkoTimeFrame], " to view chart");
		
		if(Close[0] > MathMax(dv[n][_PrevClose], dv[n][_PrevOpen])) dv[n][_CurOpen] = MathMax(dv[n][_PrevClose], dv[n][_PrevOpen]);
		else if (Close[0] < MathMin(dv[n][_PrevClose], dv[n][_PrevOpen])) dv[n][_CurOpen] = MathMin(dv[n][_PrevClose], dv[n][_PrevOpen]);
		else dv[n][_CurOpen] = Close[0];
		
		dv[n][_CurClose] = Close[0];
				
		if(dv[n][_UpWick] > dv[n][_PrevHigh]) dv[n][_CurHigh] = dv[n][_UpWick];
		if(dv[n][_DnWick] < dv[n][_PrevLow]) dv[n][_CurLow] = dv[n][_DnWick];
      
		FileWriteInteger(exti[n][_HstHandle], iv[n][_PrevTime], LONG_VALUE);		// Time
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurOpen], DOUBLE_VALUE);    // Open
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurLow], DOUBLE_VALUE);		// Low
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurHigh], DOUBLE_VALUE);		// High
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurClose], DOUBLE_VALUE);	// Close
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurVolume], DOUBLE_VALUE);	// Volume				
		FileFlush(exti[n][_HstHandle]);
            
		UpdateAllChartWindow(exti, extb, n);
		
		return(0);
 		// End historical data / Init		
	} 		
	//----------------------------------------------------------------------------
 	// HstHandle not < 0 so we always enter here after history done
	// Begin live data feed
   			
	dv[n][_UpWick] = MathMax(dv[n][_UpWick], Bid);
	dv[n][_DnWick] = MathMin(dv[n][_DnWick], Bid);

	dv[n][_CurVolume]++;   			
	FileSeek(exti[n][_HstHandle], iv[n][_LastFPos], SEEK_SET);

 	//-------------------------------------------------------------------------	   				
 	// up box	   				
   	if(Bid > dv[n][_PrevHigh]+dv[n][_BoxPoints] || CompareDoubles(Bid, dv[n][_PrevHigh]+dv[n][_BoxPoints])) {
		dv[n][_PrevHigh] = dv[n][_PrevHigh] + dv[n][_BoxPoints];
		dv[n][_PrevLow] = dv[n][_PrevLow] + dv[n][_BoxPoints];
  		dv[n][_PrevOpen] = dv[n][_PrevLow];
  		dv[n][_PrevClose] = dv[n][_PrevHigh];
  				  			
		FileWriteInteger(exti[n][_HstHandle], iv[n][_PrevTime], LONG_VALUE);
		FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevOpen], DOUBLE_VALUE);

		if (extb[n][_ShowWicks] && dv[n][_DnWick] < dv[n][_PrevLow]) FileWriteDouble(exti[n][_HstHandle], dv[n][_DnWick], DOUBLE_VALUE);
		else FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevLow], DOUBLE_VALUE);
		  			
		FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevHigh], DOUBLE_VALUE);
		FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevClose], DOUBLE_VALUE);
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurVolume], DOUBLE_VALUE);
      FileFlush(exti[n][_HstHandle]);
  	  	iv[n][_LastFPos] = FileTell(exti[n][_HstHandle]);   // Remeber Last pos in the file				  							
      	
		if(iv[n][_PrevTime] < TimeCurrent()) iv[n][_PrevTime] = TimeCurrent();
		else iv[n][_PrevTime]++;
            		
  		dv[n][_CurVolume] = 0;
		dv[n][_CurHigh] = dv[n][_PrevHigh];
		dv[n][_CurLow] = dv[n][_PrevHigh];  
		
		dv[n][_UpWick] = 0;
		dv[n][_DnWick] = EMPTY_VALUE;		
		
		UpdateAllChartWindow(exti, extb, n);
  	}
 	//-------------------------------------------------------------------------	   				
 	// down box
	else if(Bid < dv[n][_PrevLow]-dv[n][_BoxPoints] || CompareDoubles(Bid,dv[n][_PrevLow]-dv[n][_BoxPoints])) {
  		dv[n][_PrevHigh] = dv[n][_PrevHigh] - dv[n][_BoxPoints];
  		dv[n][_PrevLow] = dv[n][_PrevLow] - dv[n][_BoxPoints];
  		dv[n][_PrevOpen] = dv[n][_PrevHigh];
  		dv[n][_PrevClose] = dv[n][_PrevLow];
  				  			
		FileWriteInteger(exti[n][_HstHandle], iv[n][_PrevTime], LONG_VALUE);
		FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevOpen], DOUBLE_VALUE);
		FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevLow], DOUBLE_VALUE);

		if(extb[n][_ShowWicks] && dv[n][_UpWick] > dv[n][_PrevHigh]) FileWriteDouble(exti[n][_HstHandle], dv[n][_UpWick], DOUBLE_VALUE);
		else FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevHigh], DOUBLE_VALUE);
				  			
		FileWriteDouble(exti[n][_HstHandle], dv[n][_PrevClose], DOUBLE_VALUE);
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurVolume], DOUBLE_VALUE);
      FileFlush(exti[n][_HstHandle]);
  	  	iv[n][_LastFPos] = FileTell(exti[n][_HstHandle]);   // Remeber Last pos in the file				  							
      	
		if(iv[n][_PrevTime] < TimeCurrent()) iv[n][_PrevTime] = TimeCurrent();
		else iv[n][_PrevTime]++;      	
            		
  		dv[n][_CurVolume] = 0;
		dv[n][_CurHigh] = dv[n][_PrevLow];
		dv[n][_CurLow] = dv[n][_PrevLow];  
		
		dv[n][_UpWick] = 0;
		dv[n][_DnWick] = EMPTY_VALUE;		
		
		UpdateAllChartWindow(exti, extb, n);
     	} 
 	//-------------------------------------------------------------------------	   				
   	// no box - high/low not hit				
	else {
		if(Bid > dv[n][_CurHigh]) dv[n][_CurHigh] = Bid;
		if(Bid < dv[n][_CurLow]) dv[n][_CurLow] = Bid;
		
      if (true) dv[n][_CurOpen] = dv[n][_PrevClose];                          // shorter CurOpen
      else {                                                                  // ver 3.2	 redundant code
		 if(dv[n][_PrevHigh] <= Bid) dv[n][_CurOpen] = dv[n][_PrevHigh];
		 else if(dv[n][_PrevLow] >= Bid) dv[n][_CurOpen] = dv[n][_PrevLow];
		 else dv[n][_CurOpen] = Bid;
      }
		dv[n][_CurClose] = Bid;
		
		FileWriteInteger(exti[n][_HstHandle], iv[n][_PrevTime], LONG_VALUE);		// Time
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurOpen], DOUBLE_VALUE);    // Open
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurLow], DOUBLE_VALUE);		// Low
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurHigh], DOUBLE_VALUE);		// High
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurClose], DOUBLE_VALUE);	// Close
		FileWriteDouble(exti[n][_HstHandle], dv[n][_CurVolume], DOUBLE_VALUE);	// Volume				
      FileFlush(exti[n][_HstHandle]);
      
		UpdateAllChartWindow(exti, extb, n);
     	}
//     	return(0);            // redundant
}

// update all offline charts for Renko TF n
// modify ChartOpened
void UpdateAllChartWindow(int  exti[][],           // extern int parameters    RenkoBoxSize , RenkoBoxOffset , RenkoTimeFrame , HstHandle
                          bool& extb[][],          // extern bool parameters   ShowWicks , EmulateOnLineChart , StrangeSymbolName , ChartOpened
                          int n)                   // number of Renko TF
{
   int hwnd,len;
   string namchart = StringConcatenate(SymbolName, ",M", exti[n][_RenkoTimeFrame], " (offline)");      // name of the chart
   string gwstr = "                                                                                                                       ";    // name of the offline
   extb[n][_ChartOpened] = false;
   hwnd = WindowHandle(SymbolName, exti[n][_RenkoTimeFrame]);      //This is non-zero if at least one matching offline window exists
   if (hwnd != 0) {
      hwnd=GetParent(hwnd);
      hwnd = GetWindow(hwnd,GW_HWNDFIRST);                        // first match window 
      while (hwnd !=0) {
         len = GetWindowTextA(hwnd,gwstr,80);                     // name of the chart
//Print("GW_HWNDNEXT : ", hwnd, "    len : ",len, "    namchart : >",namchart, "<", "    gwstr : >",gwstr,"<");
         if (gwstr == namchart) {                                 // select right charts only
            extb[n][_ChartOpened] = true;                         // offline chart exists
            if(extb[n][_EmulateOnLineChart])                                                                //  emulate the user selecting the "Refresh" menu item - reprocessing of the history
               if (PostMessageA(hwnd, WM_COMMAND, CHART_CMD_UPDATE_DATA, 0) == 0) hwnd = 0;                 // window dismiss
            if (hwnd != 0 && SendTick && MT4InternalMsg != 0) PostMessageA(hwnd, MT4InternalMsg, 2, 1);     // Internal Message simulate an incoming tick
         }   
         hwnd = GetWindow(hwnd,GW_HWNDNEXT);
      }
   }
}

// describe offline charts and sets global variables
void com(int exti[][],           // extern int parameters  RenkoBoxSize , RenkoBoxOffset , RenkoTimeFrame , HstHandle
         bool extb[][])          // extern bool parameters   ShowWicks , EmulateOnLineChart , ChartOpened
{
   string c,tf,s,g;
   c = "";
   for (int i=0;i<N;i++) {
      c = c + "RenkoLiveChart(" + exti[i][_RenkoBoxSize] + " pips)     ";
      tf = "M" + exti[i][_RenkoTimeFrame];
      g = base_glob + tf;                            // name of global variable
      if (extb[i][_ChartOpened]) {
         GlobalVariableSet(g, 1);            // offline chart is refreshing
         s = "Chart  " + SymbolName + " " + tf + "  is opened offline and refreshing HST";
      } else  {
         GlobalVariableSet(g, 0);            // offline chart is not refreshing
         s = "Open Offline  " + SymbolName + " " + tf + "  to view the chart and start refreshing HST";    // + tf;
      }   
      c = c + s + "\n";
   }
   Comment(c);
}


